package cn.com.qmth.examcloud.ws.starter.api.controller;

import javax.websocket.Session;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.WithoutStackTrace;
import cn.com.qmth.examcloud.ws.starter.core.MessageOut;
import cn.com.qmth.examcloud.ws.starter.core.SessionInfo;
import cn.com.qmth.examcloud.ws.starter.core.WebSocketHelper;

@RestController
@RequestMapping("api/ctr/ws")
public class WebSocketController extends ControllerSupport {

	@WithoutStackTrace
	@PostMapping("{path}")
	public String post(@PathVariable String path, @RequestBody String message) {

		User accessUser = getAccessUser();

		Session session = WebSocketHelper.getSession(path, accessUser.getKey());

		if (null == session) {
			throw new StatusException("100001", "no ws session about path [" + path + "]");
		}

		SessionInfo sessionInfo = WebSocketHelper.getSessionInfo(session);

		if (null == sessionInfo) {
			throw new StatusException("100002", "no ws session info");
		}

		MessageOut out = new MessageOut(path, sessionInfo.getSessionId());

		JsonElement jsonEl = null;
		if (StringUtils.isNotBlank(message)) {
			try {
				jsonEl = JsonParser.parseString(message);
			} catch (JsonSyntaxException e) {
				throw new StatusException("100003", "message is not a json string");
			}
		}

		out.setContent(jsonEl);

		try {
			ThreadContext.put("TRACE_ID", sessionInfo.getSessionId());
			WebSocketHelper.sendText(session, path, out);
		} finally {
			ThreadContext.put("TRACE_ID", ThreadLocalUtil.getTraceId());
		}

		String json = JsonUtil.toJson(out);
		return json;
	}

}
